package com.lwby.marketing.controller;


import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.lwby.marketing.notify.Media;
import com.lwby.marketing.util.CacheKeyUtils;
import com.lwby.marketing.util.ResultConstant;
import com.lwby.marketing.vo.ClientInfo;
import com.lwby.marketing.vo.DeliveryDeviceInfo;
import com.lwby.marketing.vo.NovelAction;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Arrays;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@RestController
@Slf4j
@RequestMapping(value = "/clickback")
public class ClickbackService extends ServiceHelper{

    final static ImmutableSet<String> filter = ImmutableSet.of("click.special.param:00000000-0000-0000-0000-000000000000","9f89c84a559f573636a47ff8daed0d33","__IMEI__","null","NULL");

    final static ImmutableMap<String, Integer> RESULT_SUCCESS = ImmutableMap.of("status", ResultConstant.CALL_BACK_SUCCESS);

    final static int bookIdExpireTime = 604800;

    @Resource
    public RedisTemplate<String, String> redisTemplate;

    @Resource(name = "storyKafka")
    public KafkaTemplate<String, String> storyKafkaTemplate;

    private static final String UAPRE = "Mozilla/5.0 (iPhone; CPU iPhone OS ";

    private static final String UASUF = " like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148";


    @RequestMapping("/jrtt")
    @ResponseBody
    public Object jrttClick(@RequestParam Map<String, String> params){
        NovelAction action = fullJrtt(params);

        for(DeviceType type: DeviceType.values()){
            String deviceIdKey = type.getDeviceId(action.getClientInfo());
            if(null != deviceIdKey){
                DeliveryDeviceInfo ddi = action.getDeliveryDeviceInfo();

                if (StringUtils.isNotEmpty(ddi.getDj_channel())) {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getChannelByDeviceIdKeyAndPlatformId(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(Integer.parseInt(ddi.getDj_channel())), bookIdExpireTime, TimeUnit.SECONDS);
                    if (StringUtils.isNumeric(ddi.getBook_id())) {
                        redisTemplate.opsForValue().set(CacheKeyUtils.getDeliveryBookIdCacheKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi.getBook_id()), bookIdExpireTime, TimeUnit.SECONDS);
                        if (StringUtils.isNumeric(ddi.getPartId())) {
                            redisTemplate.opsForValue().set(CacheKeyUtils.getDeliveryPartIdCacheKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi.getPartId()), bookIdExpireTime, TimeUnit.SECONDS);
                        }
                    }
                }

                if (type == DeviceType.IP) {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getClickKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi),bookIdExpireTime, TimeUnit.SECONDS);
                } else {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getClickKeyByIdfaAndPlatformId(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi),bookIdExpireTime, TimeUnit.SECONDS);
                }
            }
        }

        log.info("jrttClick param,params={}",JSON.toJSONString(params));

        return RESULT_SUCCESS;
    }

    @RequestMapping("/weChat")
    @ResponseBody
    public Object wechatClick(@RequestParam Map<String, String> params){
        NovelAction action = fullWeChat(params);

        for(DeviceType type: DeviceType.values()){
            String deviceIdKey = type.getDeviceId(action.getClientInfo());
            if(null != deviceIdKey){
                DeliveryDeviceInfo ddi = action.getDeliveryDeviceInfo();

                if (StringUtils.isNotEmpty(ddi.getDj_channel())) {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getChannelByDeviceIdKeyAndPlatformId(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(Integer.parseInt(ddi.getDj_channel())), bookIdExpireTime, TimeUnit.SECONDS);
                    if (StringUtils.isNumeric(ddi.getBook_id())) {
                        redisTemplate.opsForValue().set(CacheKeyUtils.getDeliveryBookIdCacheKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi.getBook_id()), bookIdExpireTime, TimeUnit.SECONDS);
                        if (StringUtils.isNumeric(ddi.getPartId())) {
                            redisTemplate.opsForValue().set(CacheKeyUtils.getDeliveryPartIdCacheKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi.getPartId()), bookIdExpireTime, TimeUnit.SECONDS);
                        }
                    }
                }

                if (type == DeviceType.IP) {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getClickKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi),bookIdExpireTime, TimeUnit.SECONDS);
                } else {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getClickKeyByIdfaAndPlatformId(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi),bookIdExpireTime, TimeUnit.SECONDS);
                }
            }
        }
        log.info("wechatClick param,params={}",JSON.toJSONString(params));

        return RESULT_SUCCESS;
    }

    @RequestMapping("/jrttFreeVideo")
    @ResponseBody
    public Object jrttFreeVideoClick(@RequestParam Map<String, String> params){
        NovelAction action = fullJrttFreeVideo(params);

        for(DeviceVideoType type: DeviceVideoType.values()){
            String deviceIdKey = type.getDeviceId(action.getClientInfo());
            if(null != deviceIdKey){
                DeliveryDeviceInfo ddi = action.getDeliveryDeviceInfo();

                if (StringUtils.isNotEmpty(ddi.getDj_channel())) {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getChannelByDeviceIdKeyAndPlatformId(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(Integer.parseInt(ddi.getDj_channel())), bookIdExpireTime, TimeUnit.SECONDS);
                    if (StringUtils.isNumeric(ddi.getVideoResourceId())) {
                        redisTemplate.opsForValue().set(CacheKeyUtils.getDeliveryVRIdCacheKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi.getVideoResourceId()), bookIdExpireTime, TimeUnit.SECONDS);
                        if (StringUtils.isNumeric(ddi.getVideoId())) {
                            redisTemplate.opsForValue().set(CacheKeyUtils.getDeliveryVIIdCacheKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi.getVideoId()), bookIdExpireTime, TimeUnit.SECONDS);
                        }
                    }
                }

                if (type == DeviceVideoType.IDFA) {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getClickKey(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi),bookIdExpireTime, TimeUnit.SECONDS);
                } else {
                    redisTemplate.opsForValue().set(CacheKeyUtils.getClickKeyByIdfaAndPlatformId(deviceIdKey, ddi.getPlatform_id()),JSON.toJSONString(ddi),bookIdExpireTime, TimeUnit.SECONDS);
                }
            }
        }

        log.info("jrttFreeVideoClick param,params={}",JSON.toJSONString(params));

        return RESULT_SUCCESS;
    }

    public NovelAction fullJrtt(Map<String, String> params) {

        ClientInfo clientInfo = new ClientInfo();
        clientInfo.setOaid(params.get("oaid_md5"));
        clientInfo.setIdfa(params.get("idfaMd5"));
        clientInfo.setClientIp(params.get("ip"));
        clientInfo.setUa(params.get("ua"));
        //TODO;没有验证
        clientInfo.setPhoneModel(StringUtils.defaultIfEmpty(encode(params.get("model")), getModelByUa(clientInfo.getUa())));

        return getNovelAction(params,Media.NOVEL_JRTT,clientInfo);
    }

    public NovelAction fullWeChat(Map<String, String> params) {

        ClientInfo clientInfo = new ClientInfo();
        String deviceOsType = params.get("device_os_type");
        if ("ios".equals(deviceOsType)) {
            clientInfo.setIdfa(params.get("muid"));
        }
        clientInfo.setOaid(params.get("hash_oaid"));
        clientInfo.setClientIp(params.get("ip"));
        clientInfo.setUa(params.get("user_agent"));
        clientInfo.setPhoneModel(StringUtils.defaultIfEmpty(encode(params.get("model")), getModelByUa(clientInfo.getUa())));

        return getNovelAction(params,Media.NOVEL_GDT,clientInfo);
    }

    public NovelAction fullJrttFreeVideo(Map<String, String> params) {

        ClientInfo clientInfo = new ClientInfo();
        clientInfo.setOaid(params.get("oaid_md5"));
        String idfa = params.get("idfa");
        if (!filter.contains(idfa) && StringUtils.isNotEmpty(idfa)) {
            clientInfo.setIdfa(SecureUtil.md5(idfa));
        }
        clientInfo.setClientIp(params.get("ip"));
        clientInfo.setUa(params.get("ua"));
        clientInfo.setOs(params.get("os"));
        clientInfo.setPhoneModel(StringUtils.defaultIfEmpty(encode(params.get("model")), getModelByUa(clientInfo.getUa())));

        return getNovelAction(params,Media.VIDEOAPP_JRTT,clientInfo);
    }

    public NovelAction getNovelAction(Map<String, String> params,Media media,ClientInfo clientInfo){
        DeliveryDeviceInfo deliveryDeviceInfo = new DeliveryDeviceInfo();
        deliveryDeviceInfo.setClick_time(System.currentTimeMillis());
        deliveryDeviceInfo.setAd_group_id(params.get("campaign_id"));
        deliveryDeviceInfo.setMedia(media.name);
        deliveryDeviceInfo.setDj_channel(params.get("channel"));
        //短剧视频合集和视频id
        deliveryDeviceInfo.setVideoResourceId(params.get("videoResourceId"));
        deliveryDeviceInfo.setVideoId(params.get("videoId"));
        deliveryDeviceInfo.setBook_id(StringUtils.defaultIfBlank(params.get("book"), StringUtils.defaultIfBlank(params.get("books"), params.get("book_id"))));
        if (media.name.equals(Media.NOVEL_GDT)) {
            deliveryDeviceInfo.setClick_id(params.get("click_id"));
            deliveryDeviceInfo.setAd_creative_id(params.get("ad_id"));
            deliveryDeviceInfo.setAd_plan_id(params.get("adgroup_id"));
            deliveryDeviceInfo.setPlatform_id(params.get("platformId"));
            deliveryDeviceInfo.setCallback_url(params.get("callback"));
            deliveryDeviceInfo.setAdvertiser_id(params.get("account_id"));

        } else {
            deliveryDeviceInfo.setAd_creative_id(params.get("cid"));
            deliveryDeviceInfo.setAd_plan_id(params.get("adid"));
            deliveryDeviceInfo.setCallback_param(params.get("callback_param"));
            deliveryDeviceInfo.setPlatform_id(params.get("platform_id"));
            if (Objects.isNull(deliveryDeviceInfo.getCallback_param())) {
                deliveryDeviceInfo.setCallback_url(params.get("callback_url"));
            }
            deliveryDeviceInfo.setAdvertiser_id(params.get("advertiser_id"));
        }
        deliveryDeviceInfo.setUuid(UUID.randomUUID().toString());
        deliveryDeviceInfo.setAd_platform_type(params.get("ad_platform_type"));
        deliveryDeviceInfo.setTarget_audience(params.get("target_audience"));
        deliveryDeviceInfo.setIp(params.get("ip"));
        deliveryDeviceInfo.setOaid(filter.contains(params.get("oaid"))?null:params.get("oaid"));
        deliveryDeviceInfo.setModel(emptyIsNull(clientInfo.getPhoneModel()));

        //通投智选union_site
        deliveryDeviceInfo.setUnion_site(params.get("union_site"));
        deliveryDeviceInfo.setPartId(params.get("part_id"));
        //巨量广告体验版中特有的宏参，代表巨量广告体验版的广告ID
        deliveryDeviceInfo.setPromotion_id(emptyIsNull(params.get("promotion_id")));
        //巨量广告体验版中特有的宏参，代表巨量广告体验版的项目ID
        deliveryDeviceInfo.setProject_id(emptyIsNull(params.get("project_id")));
        //巨量广告体验版中的广告名称
        deliveryDeviceInfo.setPromotion_name(emptyIsNull(params.get("promotion_name")));
        //巨量广告体验版中的项目名称
        deliveryDeviceInfo.setProject_name(emptyIsNull(params.get("project_name")));
        //信息流投放广告来源(碎片 1 书籍 2)
        deliveryDeviceInfo.setSource(parseInt(params.get("source")));

        deliveryDeviceInfo.setMid1(emptyIsNull(params.get("mid1")));
        deliveryDeviceInfo.setMid2(emptyIsNull(params.get("mid2")));
        deliveryDeviceInfo.setMid3(emptyIsNull(params.get("mid3")));
        deliveryDeviceInfo.setMid4(emptyIsNull(params.get("mid4")));
        deliveryDeviceInfo.setMid5(emptyIsNull(params.get("mid5")));
        deliveryDeviceInfo.setMid6(emptyIsNull(params.get("mid6")));

        storyKafkaTemplate.send("growth_ad_click", JSONObject.toJSONString(params));

        return new NovelAction(clientInfo,deliveryDeviceInfo);
    }

    enum DeviceType {
        OAID("oaid", (c) -> isNotEmptyAndSNull(c.getOaid())?c.getOaid():null),
        IDFA("idfa",(c) -> isNotEmptyAndSNull(c.getIdfa())?c.getIdfa():null),
        IP_UA("ipua",(c) -> isNotEmptyAndSNull(c.getClientIp()) && isNotEmptyAndSNull(c.getUa())?SecureUtil.md5(c.getClientIp().concat(StringUtils.substringBefore(c.getUa(), " Chrome/"))):null),
        IP_MODEL("ipmodel",(c) -> isNotEmptyAndSNull(c.getClientIp()) && isNotEmptyAndSNull(c.getPhoneModel())?SecureUtil.md5(c.getClientIp().concat(c.getPhoneModel())):null),
        IP("ip",(c) -> isNotEmptyAndSNull(c.getClientIp())?SecureUtil.md5(c.getClientIp()):null);

        private String value;

        private Function<ClientInfo,String> fun;

        DeviceType(String value,Function<ClientInfo,String> fun) {
            this.value = value;
            this.fun = fun;
        }

        public String getValue() {
            return this.value;
        }

        public String getDeviceId(ClientInfo clientInfo){
            return fun.apply(clientInfo);
        }

        private static boolean isNotEmptyAndSNull(String str) {
            return StringUtils.isNotEmpty(str) && !filter.contains(str);
        }
    }

    enum DeviceVideoType {
        OAID("oaid", (c) -> isNotEmptyAndSNull(c.getOaid())?c.getOaid():null),
        IDFA("idfa",(c) -> isNotEmptyAndSNull(c.getIdfa())?c.getIdfa():null),
        IP_UA("ipua",(c) ->
                (isNotEmptyAndSNull(c.getClientIp()) && isNotEmptyAndSNull(c.getUa())) ?
                        (("0".equals(c.getOs()) ? SecureUtil.md5(c.getClientIp().concat(StringUtils.substringBefore(c.getUa(), " Chrome/"))) :
                                ( c.getUa().startsWith("bi kan duan ju") ? SecureUtil.md5(c.getClientIp().concat(UAPRE + Arrays.stream(c.getSystemVersion().split("\\.")).collect(Collectors.joining("_")) + UASUF)):
                                        SecureUtil.md5(c.getClientIp().concat(c.getUa()))))):null),
        IP_MODEL("ipmodel",(c) -> isNotEmptyAndSNull(c.getClientIp()) && isNotEmptyAndSNull(c.getPhoneModel())?c.getClientIp().concat(c.getPhoneModel()):null);

        private String value;

        private Function<ClientInfo,String> fun;

        DeviceVideoType(String value, Function<ClientInfo,String> fun) {
            this.value = value;
            this.fun = fun;
        }

        public String getValue() {
            return this.value;
        }

        public String getDeviceId(ClientInfo clientInfo){
            return fun.apply(clientInfo);
        }

        private static boolean isNotEmptyAndSNull(String str) {
            return StringUtils.isNotEmpty(str) && !"null".equals(str);
        }
    }
}
